% Recurrence Grammar Partitioning of Lorenz ODE

% (C) Peter beim Graben, Humboldt-Universitt zu Berlin 2013
% Reference:
% beim Graben, P. & Hutt, A. (2013). Detecting recurrence domains of
% dynamical systems by symbolic dynamics.
% Physical Review Letters, 110, 154101.

clear all
close all
clc

% integration params
% start time
ti = 0;
% end time
tf = 20;
% delta t
prec = 100;

% RP params
p = 2;      % Euclidian norm
e1 = 5.0;   % Fig. 1(a)

% epsilon test array
el = 0.2;
eh = 3.0;
es = 0.1;
epsi = el : es : eh;
 
% solve Lorenz
[t, y] = solvlorenz(ti, tf, prec, 0, 0);
y = squeeze(y);
nt = size(y, 2);

% plot Lorenz data
figure(1)
set(gcf, 'Units', 'normalized')
set(gcf, 'Position', [0.01 0.1 0.43 0.65]);

subplot(2, 1, 1)
plot(y', 'Linewidth', 1.3)
xlabel('time / samples', 'Fontsize', 14)
ylabel('x_1, x_2, x_3', 'Fontsize', 14)
axis([0 nt -30 70])
set(gca, 'FontSize', 14)
pos = get(gca, 'Position');
grid on

figure(2)
plot3(y(1, :), y(2, :), y(3, :), 'k-')
view(17,-17)
set(gca, 'FontSize', 14)
xlabel('x_1', 'Fontsize', 14)
ylabel('x_2', 'Fontsize', 14)
zlabel('x_3', 'Fontsize', 14)
hold on

% compute recurrence plot
R = recurrenceplot(y, e1, p, 1);

% visualize recurrence-based symbolic encoding
s = recgram(y, R);

% optimize ball size epsilon
opteps = maxent(y, epsi, p, 1);

% optimal recurrence-based symbolic encoding
% compute optimal recurrence plot
Ropt = recurrenceplot(y, opteps, p, 0);

% visualize optimal recurrence-based symbolic encoding
s = recgram(y, Ropt);

figure(1)
subplot(2, 1, 2)
imagesc(s)
set(gca, 'FontSize', 14)
set(gca, 'Position', pos - [0 0.2 0 0.30])
set(gca, 'Visible', 'off')
colormap('default')

% show partition
% colormap hsv
cm = colormap;
m = size(cm, 1);

figure(2)
for i = 1 : nt
    c = mod(s(i), m) + 1;
    plot3(y(1, i), y(2, i), y(3, i), 'o', ...
        'MarkerEdgeColor', cm(c, :),...
        'MarkerFaceColor', cm(c, :),...
        'MarkerSize', 2)
end

% compute symbolic recurrence plot
Rs = recurrenceplot(s, e1, 0, 1);

save('lorenz', 'y')