function opteps = maxent(data, epsilon, p, show)

% MAXENT entropy maximization for optimal ball size of recurrence grammar
% function opteps = maxent(data, epsilon, p, show)
% computes maximal rel. entropy for recurrence-based symbolic dynamics
% inputs:
% data(space, time)
% epsilon : vector of epsilon neighborhoods
% p : norm
% show?
% output : optimal epsilon

% (C) Peter beim Graben, Humboldt-Universitt zu Berlin 2013
% Reference:
% beim Graben, P. & Hutt, A. (2013). Detecting recurrence domains of
% dynamical systems by symbolic dynamics.
% Physical Review Letters, 110, 154101.

% get data soze
nt = size(data, 2);

% recurrence-based symbolic encoding
% loop through all epsilon values
for i = 1 : size(epsilon, 2)

    % compute recurrence plot for epsilon(i)
    R = recurrenceplot(data, epsilon(i), p, 0);
    % recurrence grammar
    s(i, :) = recgram(data, R);

end % of loop

% loop through all epsilon values
for k = 1 : size(epsilon, 2)

    % detect plateus
    for i = 1 : nt
        idx(i) = size(find(s(k, :) == i - 1), 2);
    end
    
    % normalize to relative frequency
    idx = idx / nt;

    % compute Shannon entropy
    h(k) = 0;
    for i = 1 : nt
        if idx(i) ~= 0
            h(k) = h(k) - idx(i) * log(idx(i));
        end
    end

    % compute relative entropy over repertoire size
    h(k) = h(k) / size(find(idx), 2);

end

% plot results
if show

    ticks = find(mod(epsilon, 0.025) == 0);
    til = num2cell(epsilon(ticks));

    figure
    colormap(hot)
    imagesc(s)
    set(gca, 'FontSize', 14)
    set(gca, 'YTick', ticks)
    set(gca, 'YTickLabel', til)
    xlabel('time / samples', 'Fontsize', 14)
    ylabel('\epsilon', 'Fontsize', 14)

    figure
    plot(epsilon, h, 'k-', 'Linewidth', 2)
    set(gca, 'FontSize', 14)
    xlabel('\epsilon', 'Fontsize', 14)
    ylabel('rel. entropy', 'Fontsize', 14)

end % of show

% output of maximal entropy and optimal 
[hmax, idxmax] = max(h);
opteps = epsilon(idxmax)

return
