function s = recgram(data, R);

% RECGRAM creates symbolic dynamics from recurrence plot.
% function s = recgram(data, R);
% creates symbolic dynamics from recurrence plot / recurrence matrix R by
% means of recurrence grammar
% input:
%
% data : multivariate time series
% R : recurrence matrix of data
%
% output:
% s : symbolic sequence of recurrence domains

% (C) Peter beim Graben, Humboldt-Universitt zu Berlin 2013
% Reference:
% beim Graben, P. & Hutt, A. (2013). Detecting recurrence domains of
% dynamical systems by symbolic dynamics.
% Physical Review Letters, 110, 154101.

% create recurrence grammar G
% get size of recurrence plot
r = size(R, 1);
% initialize first rule (l = rule number)
l = 1;
% loop through R's rows i starting at last row
for i = r : -1 : 1
    % determine for each row i of R the columns j between 1 and main diagonal
    % where R=1
    j = find(R(i, 1 : i));
    % if '1's are present in row i
    if j
        % determine smallest column index
        dest = j(1);
        % delete first entry
        j(1) = [];
        % sort vector j descending
        for k = sort(j, 'descend')
            % create rule number l by replacing column k with smallest
            % index dest
            G(l, :) = [k dest];
            % increment rule number
            l = l + 1;
        end % of k loop
    end % of if j
end % end of i loop

% initialize symbolization
% create sequence of time indices
s = 1 : r;

% get size of recurrence grammar
try
    g = size(G, 1);
catch
    % error message
    error('recgram(): couldn''t create recurrence grammar, increase epsilon!')
    return
end

% rewriting through two iterations
for k = 1 : 2
    % loop through recurrence grammar
    for i = 1 : g
        % get all occurrences of l.h.s. of rule i in string s
        idx = find(s == G(i, 1));
        % if there are any
        if idx
            % rewrite l.h.s. with r.h.s.
            s(idx) = G(i, 2);
        end % of if
    end % of rule loop
end % of recursion

% gap-less recoding
s = transsym(s);

return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sr = transsym(s);

% function sr = transsym(s);
% gap-less recoding of symbolic sequence s
% input:
% s : symbolic sequence

% get length of string s
nt = size(s, 2);

% shift string toward 0
sr = s - 1;

% check first occurences
first = 1;
while first

    % create symbol statistics
    for i = 1 : nt
        idx(i) = size(find(sr == i), 2);
    end

    % get cardinality
    ns = size(find(idx), 2);

    % new code
    newsym = find(idx == 0);
    first = newsym(find(diff(newsym) > 1)) + 1;

    % replace symbols
    new = newsym(1);
    for old = first : nt
        idxx = find(sr == old);
        if idxx
            sr(idxx) = new;
        end % of if
        new = new + 1;
    end % of loop

end % first loop

return
