function R = recurrenceplot(data, e, p, show)

% RECURRENCEPLOT creates recurrence plot.
% function R = recurrenceplot(data, e, p, show)
% creates recurrence matrix (recurrence plot) R
% from multidimensional time series data
%
% inputs:
% data(space, time) : multivariate time series / trajectory in embedding
% space
% e : ball size
% p : p-norm
% for p == 0 test for identity (for symbolic dynamics RP)
% show?
%
% output : (sparse) recurrence matrix R

% (C) Peter beim Graben, Humboldt-Universitt zu Berlin 2013
% Reference:
% beim Graben, P. & Hutt, A. (2013). Detecting recurrence domains of
% dynamical systems by symbolic dynamics.
% Physical Review Letters, 110, 154101.

% get data size
nt = size(data, 2);

% initialize sparse matrix
R = speye(nt);

for t1 = 1 : nt
    for t2 = 1 : t1 - 1
        if p == 0
            if data(:, t1) == data(:, t2)
                R(t1, t2) = 1;
                R(t2, t1) = 1;
            end
        else
            if norm(data(:, t1) - data(:, t2), p) < e
                R(t1, t2) = 1;
                R(t2, t1) = 1;
            end
        end
    end
end

if show

    figure
    set(gcf, 'Renderer', 'zbuffer')
    surface(1 - R, 'EdgeColor', 'none')
    set(gca, 'FontSize', 14)
    xlabel('time / samples', 'Fontsize', 14)
    ylabel('time / samples', 'Fontsize', 14)
    axis([1 size(R, 1) 1 size(R, 2)])
    axis square
    colormap(gray)

end

return

